#include "main.h" //We need the defines and prototypes from in.
#include "render.h"

//Some useful global handles
HINSTANCE hInst; //Will hold the current instance of the application.
HWND hWnd; // A handle to the window we will create.
HDC hDC;   // A handle to the device context of the window.

Timer *timer = NULL;

TCHAR szAppName[] = L"OpenGLES"; /*The application name and the window caption*/

TCHAR g_CurPath[MAX_PATH];

void GetExePath(HINSTANCE hInstance)
{		
		DWORD len;
		len=GetModuleFileName(hInstance, g_CurPath, 128);
		
		// Get Current Path
		while(g_CurPath[len]!='\\') len--;
		g_CurPath[len]=0;
}

extern "C" FILE* fopen_ex(const char* filename, const char* mode);

FILE* fopen_ex(const char* filename, const char* mode)
{
	if(filename[0]=='.') // if it's a relative path (starts with .\)
	{
		char path[MAX_PATH];
		sprintf(path, "%S\\%s", g_CurPath, &filename[2]);
	
		return fopen(path, mode);
	}
	else
	{
		return fopen(filename, mode);
	}
}

/*This is the WinMain function. Here we will create the rendering window, initialize OpenGL ES, write the message loop, and, at the end, clean all and release all used resources*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine,	int nCmdShow)
{
  MSG msg; //This is the message variable for the message loop
  WNDCLASS	wc; /*This structure will hold some init values for our window*/
  hInst = hInstance; // Initialization of our global variable
  bool done = false; 
	  
  /*This block of code is to ensure that the user only can run one
    instance of the application. First we search for a window with the 
    same class name, if found, we will focus it and return*/
#if 0  
  if(hWnd = FindWindow(szAppName, szAppName)) 
  {
    /* Set focus to foremost child window. The "| 0x01" is used to 
       bring any owned windows to the foreground and activate them.*/
    SetForegroundWindow((HWND)((ULONG) hWnd | 0x00000001));
    return 0;
  } 
#endif
  
  GetExePath(hInstance);


  wc.style          = CS_HREDRAW | CS_VREDRAW; /*Will force a redraw 
  if the window is resized, both horizontally or vertically*/
  wc.lpfnWndProc    = (WNDPROC) WndProc; /*this is a function pointer,
  to tell the OS what function should call when a message needs to be 
  processed*/
  wc.cbClsExtra     = 0;
  wc.cbWndExtra     = 0;
  wc.hInstance      = hInstance;
  wc.hIcon          = LoadIcon(hInstance, NULL);//Load default icon
  wc.hCursor	      = LoadCursor(NULL, IDC_HAND); // Default cursor
  wc.hbrBackground  = 0; //We don't care about the background color
  wc.lpszMenuName	  = NULL; //This application does not have a menu
  wc.lpszClassName  = szAppName; /*Important, here we must fill the
   application class name (the class name is not the same than the 
   caption of the window, but many times they are the same)*/

  //Before creating the window, we must register this new window class
  if(!RegisterClass(&wc))
    return false;
	
  hWnd=CreateWindowEx(/*WS_EX_TOPMOST*/0,
					szAppName, //Class Name
                    szAppName, //Caption string
                    /*WS_VISIBLE|*/WS_CAPTION|WS_SIZEBOX|WS_SYSMENU|WS_MAXIMIZEBOX|WS_MINIMIZEBOX,//Window style (same as WS_OVERLAPPEDWINDOW)
                    CW_USEDEFAULT,CW_USEDEFAULT,//Starting [x,y] pos.
                    400, 300,/*1920,1080,*//*CW_USEDEFAULT, CW_USEDEFAULT,*/ //Width and height
                    NULL, NULL, //Parent window and menu handle
                    hInst, NULL); /*Instance handle. Custom value to 
                    pass in the creation with the WM_CREATE message*/
	                  
  if(!hWnd) return false;
  
  //Create our timer
  timer = new Timer();
  
  if(!InitOGLES()) //OpenGL ES Initialization
  {
    MessageBox(hWnd,L"OpenGL ES init error.",L"Error",MB_OK | MB_ICONERROR);
    return false; 
  }
  
  //Bring the window to front, focus it and refresh it
  SetWindowText(hWnd, L"OpenGLES");
  if(lstrcmp(lpCmdLine, L"/f")==0) nCmdShow=SW_SHOWMAXIMIZED;
  ShowWindow(hWnd, nCmdShow); 
  UpdateWindow(hWnd);
  
  //Message Loop
  while(!done)
  {
    if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
    {
      if(msg.message==WM_QUIT)
        done = true;
	    else
      { 
	      TranslateMessage(&msg);
	      DispatchMessage(&msg);
	    }
    }
    else						
    {
	    Render();      
      timer->UpdateTimer(); //update the timer
    }
  }
  //Clean up all
  Clean();
  delete timer;
  DestroyWindow(hWnd);
  UnregisterClass(szAppName, hInst);  
  return 0;
}

extern int WindowWidth;
extern int WindowHeight;

//----------------------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) 
  {		
  //case WM_PAINT:	    
  //  ValidateRect(hWnd,NULL);
  //  return 0; 
  case WM_SIZE:
	  //if(initialized){
	  WindowWidth=LOWORD(lParam);
	  WindowHeight=HIWORD(lParam);
	  glViewport(0/*r.left*/, 0/*r.top*/, WindowWidth, WindowHeight);
	  break;
  //case WM_MOVING:
//	  OutputDebugString(L"moving\r\n");
//	  break;

  case WM_DESTROY:
    PostQuitMessage(0);
    return 0;  
  };
  return DefWindowProc(hWnd, message, wParam, lParam);   
}

